<?php

use Mockery as m;
use Myriad\Squire\SquireConfiguration;

class SquireTestCase extends PHPUnit_Framework_TestCase {

  protected $confUrl = 'https://localhost';

  protected $confToken = 'abc123';

  protected $confDomain = 'https://testcdn.example.com';

  protected $confPrefix = 'test';

  /**
   * Setup the test environment.
   */
  public function setUp() {
    global $squireConfig;
    $squireConfig = $this->getMockFullConfiguration();

    // define the config_path function
    if (!function_exists('config_path')) {
      function config_path($path) {
        return $path;
      }
    }

    // define the config function and return the default config
    if (!function_exists('config')) {
      function config() {
        global $squireConfig;

        return $squireConfig;
      }
    }
  }

  /**
   * Create a mock of the laravel cache
   *
   * @return \Mockery\MockInterface
   */
  protected function getMockCacheRepository() {
    return m::mock('Illuminate\Contracts\Cache\Repository');
  }

  /**
   * Create a mock of the guzzle client
   *
   * @return \Mockery\MockInterface
   */
  protected function getMockGuzzleClient() {
    return m::mock('GuzzleHttp\Client');
  }

  /**
   * Create a mock of the guzzle request interface
   *
   * @return \Mockery\MockInterface
   */
  protected function getMockGuzzleRequestInterface() {
    return m::mock('GuzzleHttp\Message\RequestInterface');
  }

  /**
   * Create the base configuration expected by the client
   *
   * @return array
   */
  protected function getBaseConfiguration() {
    return [
      'server_url' => $this->confUrl,
      'server_token' => $this->confToken
    ];
  }

  /**
   * Create the mock provisioning data array fetched from the server
   *
   * @return array
   */
  protected function getMockServerConfiguration() {
    return [
      'domain' => $this->confDomain,
      'prefix' => $this->confPrefix
    ];
  }

  /**
   * Create the full configuration data mock
   *
   * @return array
   */
  protected function getMockFullConfiguration() {
    return array_merge($this->getBaseConfiguration(), $this->getMockServerConfiguration());
  }

  /**
   * Create a filled configuration object
   *
   * @param null|array $config
   * @param null|\Illuminate\Contracts\Cache\Repository $repository
   * @param null|GuzzleHttp\Client $guzzle
   *
   * @return \Myriad\Squire\SquireConfiguration
   */
  protected function getConfiguration($config = null, $repository = null, $guzzle = null) {
    return new SquireConfiguration($config ?: $this->getMockFullConfiguration(), $repository ?: $this->getMockCacheRepository(), $guzzle ?: $this->getMockGuzzleClient());
  }

  /**
   * Create a new SquireClient object
   *
   * @param null|SquireConfiguration $config
   * @param null|GuzzleHttp\Client $guzzle
   *
   * @return \Myriad\Squire\SquireClient
   */
  protected function getSquire($config = null, $guzzle = null) {
    return new \Myriad\Squire\SquireClient($config ?: $this->getConfiguration(), $guzzle ?: $this->getMockGuzzleClient());
  }

  /**
   * Create a Mocked Laravel App Instance
   *
   * @return \Mockery\MockInterface
   */
  protected function getMockedApp() {
    return \Mockery::mock(new ArrayObject());
  }

}